<?php
/**
 * @package    com_responsibilities
 * @subpackage models
 * @author     Alexander Grözinger {@link http://www.agmedia.de}
 * @author     Created on Mar-2018
 * @license    GNU/GPL
 * @copyright  AGMedia.de
 */
/*
   _   ___ __  __        _ _           _
  /_\ / __|  \/  |___ __| (_)__ _   __| |___
 / _ \ (_ | |\/| / -_) _` | / _` |_/ _` / -_)
/_/ \_\___|_|  |_\___\__,_|_\__,_(_)__,_\___|
*/

// No direct access to this file
defined('_JEXEC') or die(';-)');

class ResponsibilitiesModelAssignment extends JModelAdmin
{
	
	public function save($data) {
		
		if($data['user_type'] > 0) {
			
			switch($data['user_type']) {
				case 1: // Person der Komponenten verknüpfen
					if($data['user_for_type_1'] < 1) {
						JFactory::getApplication()->enqueueMessage('Ungültige Person!', 'warning');
						return false;
					}
					$data['user_id'] = $data['user_for_type_1'];				
					break;
				case 2: //Joomla User Verknüfung
					if($data['user_for_type_2'] < 1) {
						JFactory::getApplication()->enqueueMessage('Ungültiger Joomla User!', 'warning');
						return false;
					}
					$data['user_id'] = $data['user_for_type_2'];				
					break;
					
				case 3: //Wicked User Verknüpfung
					if($data['user_for_type_3'] < 1) {
						JFactory::getApplication()->enqueueMessage('Ungültiger Wicked User!', 'warning');
						return false;
					}
					$data['user_id'] = $data['user_for_type_3'];
					break;
				default:
					return false;
					break;
			}
			
		}

		$return = parent::save($data);
		return $return;
	}
	
	/**
	 * Method to get a table object, load it if necessary.
	 *
	 * @param   string  $type    The table name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for model. Optional.
	 *
	 * @return  JTable  A JTable object
	 *
	 * @since   1.6
	 */
	public function getTable($type = 'Assignment', $prefix = 'ResponsibilitiesTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}	
	
	/**
	 * Method to get the record form.
	 *
	 * @param   array    $data      Data for the form.
	 * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
	 *
	 * @return  mixed    A JForm object on success, false on failure
	 *
	 * @since   1.6
	 */
    public function getForm($data = array(), $loadData = true)
    {
        $options = array('control' => 'jform', 'load_data' => $loadData);
        $form = $this->loadForm('responsibilities', 'assignment', $options);

        if(empty($form)) {
            return false;
        }
        
        $jinput = JFactory::getApplication()->input;
        $assignment_id = $jinput->get->get('id', 0, 'INT');
        
        $db = JFactory::getDbo();
        
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // USER_TYPE und USER_ID aus assignments Tabelle holen
        $conditions = array(
            $db->quoteName('id') 	.' = '. $db->quote($assignment_id),
        );

        $query  = $db->getQuery(true);
        $query  ->select('user_type, user_id')
            ->from($db->quoteName('#__responsibilities_assignments'))
            ->where($conditions);
        $db->setQuery($query);
        $db->execute();
        $count = $db->getNumRows();
        $user = $db->loadAssoc();

        if($user < 1) return $form;          

		$result = $this->getUserDetails($user);
		$count  = count($result);

		if ($count > 0) {

            switch($user['user_type']) {
                case 2:
                    $name = explode(' ', $result['name']);
                    $result['vorname']  = $name[0];
                    $result['nachname'] = $name[1];
                    unset($result['name']);

                    //Diese Felder bekommen wir nicht von Joomla
                    $result['strasse'] = '';
                    $result['plz'] = '';
                    $result['ort'] = '';
                    $result['email'] = '';
                    $result['telefon'] = '';
                    $result['image'] = '';
                    $result['aufgaben'] = '';
                    break;
                case 3:
                    //TODO
                    die("not yet");
                    break;
            }

			foreach($result as $field => $value) {
			    if(!empty($value)) {
				    $form->setFieldAttribute($field, 'hint', $value);
                } else {
                    $form->setFieldAttribute($field, 'hint', 'Nicht definiert');
                }
			}
		}

        return $form;
    }
    
    private function getUserDetails($user_array) {
    	
        $db = JFactory::getDbo();

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Daten für OVERRIDE Felder entsprechend des USER_TYPES holen
		$conditions = array(
		    $db->quoteName('id') 	.' = '. $db->quote($user_array['user_id']),
		);

		$query  = $db->getQuery(true);

		switch($user_array['user_type']) {
            case 1: // Person der Komponenten verknüpfen
                $query  ->select('strasse, plz, ort, email, telefon, image, aufgaben')
                        ->from($db->quoteName('#__responsibilities'))
                        ->where($conditions);
                break;
            case 2: //Joomla User Verknüfung
                $query  ->select('name, email')
                    ->from($db->quoteName('#__users'))
                    ->where($conditions);
                break;
            case 3: //Wicked User Verknüpfung
                //TODO
                break;
            default:
                return true;
                break;
        }


//        echo $query->dump(); exit;
		$db->setQuery($query);
		$db->execute();
		$count = $db->getNumRows();
		$result = $db->loadAssoc();    	
    	
    	return $result;
    }
    
    public function getItem($pk = null)
    {
    	$item = parent::getItem($pk);
    	
		$user_details = $this->getUserDetails((array) $item);
		if(!empty($user_details)) {
			$item->user_details = $user_details;	
		}
		
		return $item;
    }

	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return  mixed  The data for the form.
	 *
	 * @since   1.6
	 */
	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState(
			'com_responsibilities.edit.assignment.data',
			array()
		);

		if (empty($data))
		{
			$data = $this->getItem();
		}

		if($data->user_type > 0) {

			switch($data->user_type) {
				case 1: // Person der Komponenten verknüpfen
					$data->user_for_type_1 = $data->user_id;
					break;
				case 2: //Joomla User Verknüfung
					$data->user_for_type_2 = $data->user_id;
					break;
				case 3: //Wicked User Verknüpfung
					$data->user_for_type_3 = $data->user_id;
					break;
				default:
					break;
			}

		}
		
		// echo "<pre>"; print_r($data); exit;


		return $data;
	}

}