<?php
/**
 * @package    com_responsibilities
 * @subpackage models
 * @author     Alexander Grözinger {@link http://www.agmedia.de}
 * @author     Created on Mar-2018
 * @license    GNU/GPL
 * @copyright  AGMedia.de
 */
/*
   _   ___ __  __        _ _           _
  /_\ / __|  \/  |___ __| (_)__ _   __| |___
 / _ \ (_ | |\/| / -_) _` | / _` |_/ _` / -_)
/_/ \_\___|_|  |_\___\__,_|_\__,_(_)__,_\___|
*/

// No direct access to this file
defined('_JEXEC') or die(';-)');

/**
 * Methods supporting a list of banner records.
 *
 * @since  1.6
 */
class ResponsibilitiesModelAssignments extends JModelList
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JControllerLegacy
	 * @since   1.6
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			//TODO Filter
			$config['filter_fields'] = array(
				'id', 'a.id',
				'cid', 'a.cid',
				'vorname', 'u.vorname',
				'nachname', 'u.nachname',
				'state', 'a.state',
				'a.user_type',
				'catid', 'a.catid', 'category_name',
				'checked_out', 'a.checked_out',
				'checked_out_time', 'a.checked_out_time',
				'created', 'a.created',
				'publish_up', 'a.publish_up',
				'publish_down', 'a.publish_down',
				'published',
				'ordering', 'a.ordering'
			);
		}

		parent::__construct($config);
	}
	
	public function getItems() {
		$items = parent::getItems();

		// Daten hinzufügen
		if(!empty($items)) {
			$categories = new ResponsibilitiesCategories;
			$category_tree = array();
			
			foreach($items as $key => $item) {
				$items[$key]->category_tree = $categories->getCategoryTreeText($item->cid);
				$user_details = $this->getUserDetails($item);

                $items[$key]->vorname = $user_details['vorname'];
                $items[$key]->nachname = $user_details['nachname'];

                //User_Type schreiben
                switch($item->user_type) {
                    case 1:
						if(JFactory::getLanguage()->getTag() == 'de-DE') $text = 'User der Komponente';
						if(JFactory::getLanguage()->getTag() == 'en-GB') $text = 'User of component';
						
                        $items[$key]->user_type = $text;
                        break;
                    case 2:								
                        $items[$key]->user_type = 'Joomla User';
                        break;
                    case 3:
                        $items[$key]->user_type = 'Wicked User';
                        break;
                }
			}
		}

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Ordering für Nachname und Vorname
        $orderCol  = $this->state->get('list.ordering', 'u.nachname');
        $orderDirn = $this->state->get('list.direction', 'ASC');

        if($orderCol == 'u.nachname' || $orderCol == 'nachname' || $orderCol == 'u.vorname' || $orderCol == 'vorname') {
            //Falls ein *.* in orderCol vorkommt, den Punkt loswerden
            if(strpos($orderCol, '.')) {
                $orderCol_exploded = explode('.', $orderCol);
                $orderCol = $orderCol_exploded[1];
            }

            //Sort Flag definieren
            $sort_flag = (strtoupper($orderDirn) == 'ASC') ? 4 : 3;

            //Sortier-Array bauen
            $sort = array();
            foreach ($items as $i => $obj) {
                $sort['nachname'][$i] = $obj->{'nachname'};
                $sort['vorname'][$i] = $obj->{'vorname'};
            }

            //Sortierreihenfolge definieren und sortieren
            if($orderCol == 'nachname') {
                array_multisort($sort['nachname'], $sort_flag, $sort['vorname'], $sort_flag, $items);
            } else {
                array_multisort($sort['vorname'], $sort_flag, $sort['nachname'], $sort_flag, $items);
            }
        }

		return $items;
	}

	private function getUserDetails($item) {
        $db = JFactory::getDbo();

        $query  = $db->getQuery(true);

        switch($item->user_type) {
            case 1:
                $query->select($db->quoteName(array('vorname', 'nachname')));
                $query->from($db->quoteName('#__responsibilities'));
                $query->where($db->quoteName('id') .' = '. $db->quote($item->user_id));
                break;
            case 2:
                $query->select($db->quoteName(array('name')));
                $query->from($db->quoteName('#__users'));
                $query->where($db->quoteName('id') .' = '. $db->quote($item->user_id));
                break;
            case 3:
                //TODO
                die("not yet");
                break;
        }

        $db->setQuery($query);
        $db->execute();
        $count  = $db->getNumRows();
        $result = $db->loadAssoc();

        if ($count < 1) return array();

        $return = array();
        switch($item->user_type) {
            case 1:
                $return['vorname']  = $result['vorname'];
                $return['nachname'] = $result['nachname'];
                break;
            case 2:
                $name = explode(' ', $result['name']);
                $return['vorname']  = $name[0];
                $return['nachname'] = $name[1];
                break;
            case 3:
                //TODO
                die("not yet");
                break;
        }


        return $return;
    }

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  JDatabaseQuery
	 *
	 * @since   1.6
	 */
	protected function getListQuery()
	{
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Default query variables
        $db    = $this->getDbo();
        $query = $db->getQuery(true);
		
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				  'a.id AS id,'
				. 'a.user_type			AS user_type,'
				. 'a.user_id			AS user_id,'
				. 'a.cid		        AS cid,'
                . 'a.checked_out		AS checked_out,'
                . 'a.checked_out_time	AS checked_out_time,'
                . 'a.publish_up			AS publish_up,'
                . 'a.publish_down		AS publish_down,'
                . 'a.state				AS state,'
                . 'a.ordering			AS ordering'
			)
		);
		$query->from($db->quoteName('#__responsibilities_assignments', 'a'));

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Join over the users for the checked out user.
		$query->select($db->quoteName('uc.name', 'editor'))
			->join('LEFT', $db->quoteName('#__users', 'uc') . ' ON uc.id = a.checked_out');

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Join over the categories.
		$query->select($db->quoteName('c.title', 'category_name'))
			->join('LEFT', $db->quoteName('#__categories', 'c') . ' ON c.id = a.cid');

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Join over the persons.
        $query
            ->join('LEFT', $db->quoteName('#__responsibilities', 'u') . ' ON u.id = a.user_id');

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Filter by published state
		$published = $this->getState('filter.published');

		if (is_numeric($published))
		{
			$query->where($db->quoteName('a.state') . ' = ' . (int) $published);
		}
		elseif ($published === '')
		{
			$query->where($db->quoteName('a.state') . ' IN (0, 1)');
		}

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Filter by category.
        $categoryId = $this->getState('filter.cid');
        if (is_numeric($categoryId)) {
            ////////////////////////////////////////////////////////////////////////////////////////////////////
            // SubQuery bauen um auch Kinder der Kategorien beim Filtern zu berücksichtigen
            $subquery = $db->getQuery(true);
            $subquery
                ->select('n.title, COUNT(*) AS level, n.id as id')
                ->from('#__categories AS n, #__categories AS p')
                ->where('n.lft BETWEEN p.lft AND p.rgt')
                ->where($db->quoteName('n.extension') . ' = ' . $db->quote('com_responsibilities'))
                ->where($db->quoteName('p.id') . ' = ' . $db->quote($categoryId));

            $subquery->group('n.lft');
            $subquery->order('n.lft');
            $db->setQuery($subquery);
            $db->execute();
            $count  = $db->getNumRows();
            $result = $db->loadAssocList();

            ///////////////////////////////////////////////////////////////////////////////////////////////////
            // SubQuery Ergebnis auswerten und die Filterkriterien auf die Hauptquery anwenden
            if ($count > 0) {
                $categories = array();
                foreach ($result as $cat) {
                    $categories[] = $db->escape($cat['id']);
                }
                $categories_string = implode(', ', $categories);
                $query->where($db->quoteName('a.cid') . ' IN (' . $categories_string . ')');
            } else {
                $query->where($db->quoteName('a.cid') . ' = ' . (int)$categoryId);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Filter by search in title
		$search = $this->getState('filter.search');

		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where($db->quoteName('a.id') . ' = ' . (int) substr($search, 3));
			}
			else
			{
				$search = $db->quote('%' . str_replace(' ', '%', $db->escape(trim($search), true) . '%'));
				$query->where('(u.vorname LIKE ' . $search . ' OR u.nachname LIKE '.$search.')');
			}
		}

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Add the list ordering clause.
		$orderCol  = $this->state->get('list.ordering', 'u.nachname');
		$orderDirn = $this->state->get('list.direction', 'ASC');

		if ($orderCol == 'a.ordering' || $orderCol == 'category_title')
		{
			$orderCol = 'c.title ' . $orderDirn . ', a.ordering';
		}

		//Ordering nach Namen passiert in getItems()
		if($orderCol != 'u.nachname' && $orderCol != 'nachname' && $orderCol != 'u.vorname' && $orderCol != 'vorname') {
		    $query->order($db->escape($orderCol . ' ' . $orderDirn));
        }

		// echo $query->dump(); exit;
		
		return $query;
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  A prefix for the store id.
	 *
	 * @return  string  A store id.
	 *
	 * @since   1.6
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id .= ':' . $this->getState('filter.search');
		$id .= ':' . $this->getState('filter.published');
		$id .= ':' . $this->getState('filter.category_id');

		return parent::getStoreId($id);
	}

	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param   string  $type    The table type to instantiate
	 * @param   string  $prefix  A prefix for the table class name. Optional.
	 * @param   array   $config  Configuration array for model. Optional.
	 *
	 * @return  JTable  A JTable object
	 *
	 * @since   1.6
	 */
	public function getTable($type = 'Assignment', $prefix = 'ResponsibilitiesTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @param   string  $ordering   An optional ordering field.
	 * @param   string  $direction  An optional direction (asc|desc).
	 *
	 * @return  void
	 *
	 * @since   1.6
	 */
	protected function populateState($ordering = 'a.ordering', $direction = 'asc')
	{
		// Load the filter state.
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search', '', 'string'));
		$this->setState('filter.published', $this->getUserStateFromRequest($this->context . '.filter.published', 'filter_published', '', 'string'));
		$this->setState('filter.category_id', $this->getUserStateFromRequest($this->context . '.filter.category_id', 'filter_category_id', '', 'cmd'));

		// Load the parameters.
		$this->setState('params', JComponentHelper::getParams('com_responsibilities'));

		// List state information.
		parent::populateState($ordering, $direction);
	}
}
